@@ -19,7 +19,7 @@ use Carp;
 
 =head1 NAME
 
-Mail::DKIM::DkSignature - represents a DomainKeys-Signature header
+Mail::DKIM::DkSignature - a DomainKeys signature header
 
 =head1 CONSTRUCTORS
 
@@ -126,25 +126,6 @@ sub body_hash
 	croak "body_hash not implemented";
 }
 
-=head2 algorithm() - get or set the algorithm (a=) field
-
-The algorithm used to generate the signature.
-Defaults to "rsa-sha1", an RSA-signed SHA-1 digest.
-
-=cut
-
-sub algorithm
-{
-	my $self = shift;
-
-	if (@_)
-	{
-		$self->set_tag("a", shift);
-	}
-
-	return lc $self->get_tag("a") || 'rsa-sha1';
-}	
-
 =head2 canonicalization() - get or set the canonicalization (c=) field
 
   $signature->canonicalization("nofws");
@@ -195,9 +176,7 @@ sub domain
 
 sub expiration
 {
-	my $self = shift;
-	croak "cannot change expiration on " . ref($self) if @_;
-	return undef;
+	croak "expiration not implemented";
 }
 
 use MIME::Base64;
@@ -226,12 +205,6 @@ sub check_protocol
 	return ($protocol eq "dns");
 }
 
-sub check_version
-{
-	#DomainKeys doesn't have a v= tag
-	return 1;
-}
-
 sub get_algorithm_class
 {
 	my $self = shift;
@@ -274,59 +247,16 @@ In list context, the header field names will be returned as a list.
 #sub headerlist
 # is in Signature.pm
 
-=head2 identity() - get the signing identity
-
-  my $i = $signature->identity;
-
-In DomainKey signatures, the signing identity is the first address
-found in the Sender header or the From header. This field is
-populated by the Verifier when processing a DomainKey signature.
-
-=cut
-
 sub identity
 {
 	my $self = shift;
 	croak "cannot change identity on " . ref($self) if @_;
-	return $self->{dk_identity};
-}
-
-=head2 identity_source() - determine which header had the identity
-
-  my $source = $signature->identity_source;
-
-If the message is being verified, this method will tell you which
-of the message headers was used to determine the signature identity.
-Possible values are "header.sender" and "header.from".
-
-=cut
-
-sub identity_source
-{
-	my $self = shift;
-	croak "unexpected argument" if @_;
-	return $self->{dk_identity_source};
-}
-
-# init_identity() - initialize the DomainKeys concept of identity
-#
-# The signing identity of a DomainKeys signature is the sender
-# of the message itself, i.e. the address in the Sender/From header.
-# The sender may not be known when the signature object is
-# constructed (since the signature usually precedes the From/Sender
-# header), so use this method when you have the From/Sender value.
-# See also finish_header() in Mail::DKIM::Verifier.
-#
-sub init_identity
-{
-	my $self = shift;
-	$self->{dk_identity} = shift;
-	$self->{dk_identity_source} = shift;
+	return "@" . $self->domain;
 }
 
 sub method
 {
-	croak "method not implemented (use canonicalization instead)";
+	croak "method not implemented";
 }	
 
 =head2 protocol() - get or set the query methods (q=) field
@@ -342,9 +272,7 @@ sub protocol {
 	(@_) and
 		$self->set_tag("q", shift);
 
-	# although draft-delany-domainkeys-base-06 does mandate presence of a
-	# q=dns tag, it is quote common that q tag is missing - be merciful
-	return !defined($self->get_tag("q")) ? 'dns' : $self->get_tag("q");
+	return $self->get_tag("q");
 }	
 
 =head2 selector() - get or set the selector (s=) field
@@ -374,22 +302,4 @@ sub version
 	croak "version not implemented";
 }
 
-=head1 SEE ALSO
-
-L<Mail::DKIM::Signature> for DKIM-Signature headers
-
-=head1 AUTHOR
-
-Jason Long, E<lt>jlong@messiah.eduE<gt>
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2006-2007 by Messiah College
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.6 or,
-at your option, any later version of Perl 5 you may have available.
-
-=cut
-
 1;
